import clr
clr.AddReferenceByName('FarseerPhysics, Version=2.1.1.0, Culture=neutral, PublicKeyToken=null')
from FarseerGames.FarseerPhysics import PhysicsSimulator
from FarseerGames.FarseerPhysics.Factories import BodyFactory
from FarseerGames.FarseerPhysics.Factories import GeomFactory
from FarseerGames.FarseerPhysics.Factories import SpringFactory
from FarseerGames.FarseerPhysics.Mathematics import Vector2
from FarseerGames.FarseerPhysics.Dynamics.Springs import FixedLinearSpring

from System import Random

from models.rectangle import Rectangle
from models.wall import Wall
from models.spring import Spring
from models.circle import Circle

class Model:

  def __init__(self):
    
    self.simulator = PhysicsSimulator(Vector2(0, 100))
    self.simulator.BiasFactor = 0.4

    self.walls = []    
    self.rectangles = []
    self.circles = []

    self.setup()
    
  def create_pyramid(self,row,start,func):
    space = 90
    for i in range(row):
      func(Vector2(start.X+i*space-(row*space*.5) ,start.Y+row*60))
   
    if (row > 1):
      self.create_pyramid(row - 1,start,func)
 
  def create_random(self, point):
    
    if self.random.Next(2)==1:
      self.create_circle(point)
    else:
      self.create_rectangle(point)
 
 
  def create_rectangle(self,point):
 
    rect = Rectangle(self.simulator, Vector2(50,50),point) 
    self.rectangles.append(rect)

    
  def create_circle(self,point):
    
    circle = Circle(self.simulator, Vector2(80,50),point) 
    self.circles.append(circle)  
    
    
  def setup(self):

    self.random = Random()

    self.create_pyramid(4,Vector2(300,0),self.create_random)

    self.create_border(Vector2(0,0),Vector2(500,300))
    
    # spring not yet attached yet  
    self.spring = None

    
  def create_border(self,p1, p2):
    
    # parametized create walls
    border = 20
    
    self.walls.append(Wall(self.simulator,p1.X-border,p2.Y,p2.X+border,p2.Y+border)) #bottom
    self.walls.append(Wall(self.simulator,p1.X-border,p1.Y-border,p2.X+border,p1.Y)) #top
    self.walls.append(Wall(self.simulator,p1.X-border,p1.Y,p1.X,p2.Y)) #left
    self.walls.append(Wall(self.simulator,p2.X,p1.Y,p2.X+border,p2.Y)) #right
 
    
  def update(self): 

    self.simulator.Update(0.01)


  def mouse_move(self,point):  

    if not self.spring == None:
      self.spring.attach(Vector2(point.X,point.Y))

      
  def mouse_click(self,point):

    point = Vector2(point.X,point.Y)
    geom = self.simulator.Collide(point);
    if not geom == None:
      self.spring = Spring(self.simulator, geom, point);


  def mouse_release(self,point):

    if not self.spring == None:
      self.spring.dispose()
      self.spring = None

  
  def mouse_cancel(self,point):

    if not self.spring == None:
      self.spring.dispose()
      self.spring = None
