from System.Windows.Markup import XamlReader
from System.Windows.Media.Animation import Storyboard
from System.Windows.Media import SolidColorBrush
from System.Windows.Media import Colors

from views.spring import Spring  
from views.rectangle import Rectangle
from views.wall import Wall
from views.circle import Circle

class View:

  def __init__(self, model):

    self.load('main.xaml')
    self.model = model
    
    self.control.MouseLeftButtonDown += self.mouse_click
    self.control.MouseLeftButtonUp += self.mouse_release
    self.control.MouseLeave += self.mouse_leave
    self.control.MouseMove += self.mouse_move

    # setup timer
    self.timer = Storyboard()
    self.control.Resources.Add("timer",self.timer)
    self.timer.Completed += self.frame
    self.timer.Begin()

    self.blocks = []

    self.spring = Spring(self.control)
    
    for r in self.model.rectangles:
      rect = Rectangle(self.control, r)
      #rect.control.brush.Color = Colors.Blue
      self.blocks.append(rect)
      rect.update()

    for model in self.model.circles:
      circle = Circle(self.control, model)
      #circle.control.brush.Color = Colors.Red
      self.blocks.append(circle)
      circle.update()
      
    for w in self.model.walls:
      rect = Wall(self.control, w)  
      rect.update()
  
    
  def load(self, url):
  
    xaml = open(url).read()
    self.control = XamlReader.Load(xaml)

 
    
  def frame(self, *args):

    self.model.update()

    for block in self.blocks:
      block.update()

    if not self.model.spring == None:
      self.spring.show(self.model.spring)
    else: 
      self.spring.hide()

    self.timer.Begin()
    
    
  def mouse_click(self, sender, e):
    #debug.Text = e.GetPosition(self.control).ToString()
    self.model.mouse_click(e.GetPosition(self.control))   


  def mouse_release(self, sender, e):

    self.model.mouse_release(e.GetPosition(self.control))


  def mouse_move(self, sender, e):

    self.model.mouse_move(e.GetPosition(self.control))
    
    
  def mouse_leave(self, sender, e):

    self.model.mouse_cancel(e.GetPosition(self.control))
    
    